<?php

namespace Drupal\Tests\search_api_meilisearch\Kernel\Backend;

use Drupal\entity_test\Entity\EntityTest;
use Drupal\search_api\Item\Field;

/**
 * Provides tests when using reserved names as index fields.
 *
 * @group search_api_meilisearch
 */
class ReservedIndexFieldsTest extends BackendKernelTestBase {

  protected const TEST_INDEX = 'kernel_index_fields_test_index';

  /**
   * Tests searching with reserved search_api_document_id field in index fields.
   */
  public function testSearchWithIdFieldInIndex(): void {
    $this->prepareTestIndex(self::TEST_INDEX);

    $fields = $this->index->getFields();
    $fields['search_api_document_id'] = new Field($this->index, 'search_api_document_id');
    $fields['search_api_document_id']->setPropertyPath('body')
      ->setDatasourceId('entity:entity_test')
      ->setType('string');
    $this->index->setFields($fields);
    $this->index->save();
    $this->prepareItems();

    $results = $this->index->query()->range()->execute();
    $this->assertEquals(5, $results->getResultCount());
  }

  /**
   * Tests searching with reserved "search_api_id" field in index fields.
   */
  public function testSearchWithItemIdFieldInIndex(): void {
    $this->prepareTestIndex(self::TEST_INDEX);

    $fields = $this->index->getFields();
    $fields['search_api_id'] = new Field($this->index, 'search_api_id');
    $fields['search_api_id']->setPropertyPath('id')
      ->setDatasourceId('entity:entity_test')
      ->setType('integer');
    $this->index->setFields($fields);
    $this->index->save();

    $this->prepareItems();

    $results = $this->index->query()->range()->execute();
    $this->assertEquals(5, $results->getResultCount());
  }

  /**
   * Prepares and indexes test entities.
   */
  protected function prepareItems(): void {
    foreach (range(1, 5) as $key => $weight) {
      EntityTest::create([
        'name' => 'Test entity ' . $key,
        'body' => 'This is the test body',
        'weight' => $weight,
      ])->save();
    }

    $this->index->indexItems();
  }

  /**
   * {@inheritdoc}
   */
  protected function getTestIndexes(): array {
    return [self::TEST_INDEX];
  }

}
