<?php

namespace Drupal\Tests\search_api_meilisearch\Kernel\Backend;

use Drupal\search_api\Item\Field;

/**
 * Provides tests for sorting and ranking rules settings.
 *
 * @group search_api_meilisearch
 */
class SortAndRankingRulesSettingsTest extends BackendKernelTestBase {

  protected const TEST_INDEX = 'kernel_sort_ranking_test_index';

  /**
   * Tests sortable attributes and ranking rules on index operations.
   */
  public function testRankingRulesAndSortingAttributesAreSet(): void {
    $this->prepareTestIndex(self::TEST_INDEX);

    // Tests ranking and sorting settings are set after index creation.
    $defaultRankingRules = [
      'sort',
      'words',
      'attribute',
      'typo',
      'proximity',
      'exactness',
    ];
    $settings = $this->apiService->getRankingRules(self::TEST_INDEX);
    $this->assertEquals($defaultRankingRules, $settings);

    $defaultSortingAttributes = [
      'body',
      'name',
      'weight',
    ];

    $settings = $this->apiService->getSortableAttributes(self::TEST_INDEX);
    $this->assertEquals($defaultSortingAttributes, $settings);

    // Tests that sorting settings are set after adding a field to index.
    $idField = new Field($this->index, 'id');
    $idField->setType('integer')
      ->setDatasourceId('entity:entity_test')
      ->setPropertyPath('id');

    $this->index
      ->addField($idField)
      ->save();

    $defaultSortingAttributes = ['body', 'id', 'name', 'weight'];

    $settings = $this->apiService->getSortableAttributes(self::TEST_INDEX);
    $this->assertEquals($defaultSortingAttributes, $settings);

    $this->index
      ->removeField('id')
      ->save();

    $defaultSortingAttributes = ['body', 'name', 'weight'];
    $settings = $this->apiService->getSortableAttributes(self::TEST_INDEX);
    $this->assertEquals($defaultSortingAttributes, $settings);
  }

  /**
   * {@inheritdoc}
   */
  protected function getTestIndexes(): array {
    return [self::TEST_INDEX];
  }

}
